/* Programme qui sert à tester le circuit imprimé principal qui supporte la carte
   Arduino NANO et les composants périphériques principaux.
   Ce circuit imprimé est conforme à l'implantation des composants donné en
   chapitre "Les affectations des broches de la carte Arduino NANO" du didacticiel.
   UTILISATION :
   -------------
   • Le codeur incrémental est pris en compte à la fin de chaque cycle et incrémente
     ou décrémente "Compteur" en fonction du sens de rotation.
   • Le BP central du codeur rotatif enfoncé allume les trois LEDs à sa prise en compte.
     génère un petit BIP sur le Buzzer. Puis à son relacher les trois LEDs sont
     éteintes et le cycle recommence.
   • Durant un cycle les 9 pilotages des inducteurs ainsi que les trois LEDs sont
     passés à l'état "1" durant [Impulsion] milli-secondes.
   • Maintenir le BP TIR ou le BP marche enfoncé vera son état affiché sur OLED.
   • En dernière ligne d'écran le nombre de cycles effectué est affiché, simple
     complément sur la page écran.
   
Branchements à effectuer :
//-------------------------- Entrées Analogiques -----------------------------
 A0 : Pilotage de la LED verte.
 A1 : Pilotage de la LED rouge.
 A2 : Pilotage de la LED bleue.
 A3 : Bouton central du codeur rotatif.
 A4 : SDA OLED.
 A5 : SCL OLED.
 A6 : B.P. MARCHE.
 A7 : B.P. Tir.
//----------------------- Entrées / Sorties BINAIRES ------------------------- 
 Codeur rotatif B : Broche D2. >>>>>>>>>>> ATTENTION : En fonction du codeur KY-40 approvisionné il
 Codeur rotatif A : Broche D3. >>>>>>>>>>>>>>>>>>>>>>> faudra éventuellement inverser D2 et D3 pour
                               >>>>>>>>>>>>>>>>>> obtenir un sens de variation correct.
 D2  : A du Codeur rotatif.
 D3  : B du Codeur rotatif.
 D4  : Pilotage de la bobine B1.
 D5  : Pilotage de la bobine B2.
 D6  : Pilotage de la bobine B3.
 D7  : Pilotage de la bobine B4.
 D8  : Pilotage de la bobine B5.
 D9  : Pilotage de la bobine B6.
 D10  : Pilotage de la bobine B7.
 D11  : Pilotage du bruiteur passif.
 D12  : Pilotage de la bobine B8.
 D13  : Pilotage de l'inducteur du collisionneur.
 
                                                   PGM               RAM dynamique
Compilateur Version 1.7.9                 13230 Octets (43%)     323 Octets (15%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // Usage de la ligne I2C (Two Wire Interface)
//------------- Définition des broches d'entrées analogiques ----------------
#define VERTE 14 // A0 pour piloter la LED Verte.
#define ROUGE 15 // A1 pour piloter la LED Rouge.
#define BLEUE 16 // A2 pour piloter la LED Bleue.
#define BP_central 17 // A3 pour lire le bouton central.
#define BP_Marche 20 // A6 pour lire le BP Marche.
#define BP_Tir 21 // A7 pour lire le BP Tir.

//-------------- Définition des broches de sorties Binaires -----------------
#define Codeur_B 2   // D2 : Codeur rotatif.
#define Codeur_A 3   // D3 : Codeur rotatif.
#define Bobine1 4    // D4 : Pilote B1.
#define Bobine2 5    // D5 : Pilote B2.
#define Bobine3 6    // D6 : Pilote B3.
#define Bobine4 7    // D7 : Pilote B4.
#define Bobine5 8    // D8 : Pilote B5.
#define Bobine6 9    // D9 : Pilote B6.
#define Bobine7 10   // D10 : Pilote B7.
#define Buzzer  11   // D11 : Bruiteur passif.
#define Bobine8 12   // D12 : Pilote B8.
#define TIR 13       // D13 : Pilote le collisionneur.

#define Impulsion 20 // Durée d'éclairage des LEDs et de commutation des inducteurs.

//-------------------------- Variables du programme -------------------------
boolean A_set, B_set;   // État des sorties déphasées du codeur.
boolean BP_central_a_traiter;
boolean INT_active;     // Indique un INT_active d'état détecté.
boolean Sens_positif; // Sens de rotation.

byte Compteur; // Simple compteur pour tester le codeur rotatif.

byte NB_cycles; // Compteur de cycles effectués depuis le RESET.

float Frequence, T0, T1;

volatile boolean Rotation;  // Indique une activation du codeur rotatif.
//---------------------------------------------------------------------------

void setup() {
  u8g.setFont(u8g_font_6x10); // Police de matrices de 6x10 pixels lettres MJA/min comprises.
  //---------------------- Définition des vecteurs d'interruption ----------------------
  analogReference(DEFAULT);
  attachInterrupt(0, Front_sur_A, CHANGE); // Appel à Front_sur_A si INT_active d'état détecté.
  attachInterrupt(1, Front_sur_B, CHANGE); // Appel à Front_sur_B si INT_active d'état détecté.
  u8g.setFont(u8g_font_profont12); // Police de matrices de 6x10 pixels lettres MAJ/min comprises.
  pinMode(Codeur_A,INPUT); pinMode(Codeur_B,INPUT);
  pinMode(VERTE, OUTPUT); digitalWrite(VERTE, LOW);
  pinMode(ROUGE, OUTPUT); digitalWrite(ROUGE, LOW);  
  pinMode(BLEUE, OUTPUT); digitalWrite(BLEUE, LOW);
  pinMode(Bobine1, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine2, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine3, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine4, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine5, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine6, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine7, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(Bobine8, OUTPUT); digitalWrite(Bobine1, LOW);
  pinMode(TIR, OUTPUT); digitalWrite(TIR, LOW);
  //--------------------- Initialisation des variables ---------------------
  Compteur = 0; NB_cycles = 0;
  BP_central_a_traiter = false; Rotation = false;
  Front_sur_A(); Front_sur_B();} // Voir informations en fin de listage.

void loop() {
  NB_cycles++; if (Rotation)TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL();
  //======================= Boucle de commutation =======================
  digitalWrite(Bobine1, HIGH); delay(Impulsion); digitalWrite(Bobine1, LOW);
  digitalWrite(Bobine2, HIGH); delay(Impulsion); digitalWrite(Bobine2, LOW);
  digitalWrite(Bobine3, HIGH); delay(Impulsion); digitalWrite(Bobine3, LOW);
  digitalWrite(Bobine4, HIGH); delay(Impulsion); digitalWrite(Bobine4, LOW);
  digitalWrite(Bobine5, HIGH); delay(Impulsion); digitalWrite(Bobine5, LOW);
  digitalWrite(Bobine6, HIGH); delay(Impulsion); digitalWrite(Bobine6, LOW);
  digitalWrite(Bobine7, HIGH); delay(Impulsion); digitalWrite(Bobine7, LOW);
  digitalWrite(Bobine8, HIGH); delay(Impulsion); digitalWrite(Bobine8, LOW);
  digitalWrite(TIR, HIGH); delay(Impulsion); digitalWrite(TIR, LOW);
  digitalWrite(BLEUE, HIGH); delay(Impulsion); digitalWrite(BLEUE, LOW);
  digitalWrite(VERTE, HIGH); delay(Impulsion); digitalWrite(VERTE, LOW);
  digitalWrite(ROUGE, HIGH); delay(Impulsion); digitalWrite(ROUGE, LOW);
  Teste_activation_BP_central();
  Affiche_PAGE_OLED();}

//-------------------- Routines de servitude pour traiter le capteur incrémental -------------------
  void Delay_antirebonds() {for(int T=0; T<1000; T++);} // Valeur de 1000 expérimentale.
  
  void Front_sur_A(){ // Interruption quand A change d'état avant B.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds.
    if (digitalRead(Codeur_A) != A_set ) {// Test pour savoir si l'état a changé.
      A_set = !A_set;
      if (A_set && !B_set) {Sens_positif = true; Rotation = true; Delay_antirebonds();} } }
  
  void Front_sur_B(){ // Interruption quand B change d'état avant A.
    if (INT_active) Delay_antirebonds(); // Attendre un peu la fin des rebonds. 
    if( digitalRead(Codeur_B) != B_set ) {// Test pour savoir si l'état a changé.
      B_set = !B_set;
      // Décrémenter le compteur si B avant A.
      if (B_set && !A_set) {Sens_positif = false; Rotation = true; Delay_antirebonds();} } }
  //--------------------------------------------------------------------------------------------------
  
void Affiche_PAGE_OLED() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  CADRES();
  Aff_TEXTE(27, 10 ,"Compteur = ");  u8g.print(Compteur);
  Aff_TEXTE(25,27,"Bp Marche : ");
  if (analogRead(BP_Marche) < 512) u8g.print("OUI"); else u8g.print("NON");
  Aff_TEXTE(43,41,"Bp Tir : ");
  if (analogRead(BP_Tir) < 512) u8g.print("OUI"); else u8g.print("NON");
  Aff_TEXTE(25,55, "Nb cycles : "); u8g.print(NB_cycles);}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
 
void CADRES() {u8g.drawFrame(0,0,128,14); u8g.drawFrame(0,16,128,48);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {
  char Caractere; byte I = 0;
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}

void Teste_activation_BP_central() {
  if (analogRead(BP_central) < 512) {// Si le B.P. central est activé :
     digitalWrite(BLEUE, HIGH); digitalWrite(VERTE, HIGH); digitalWrite(ROUGE, HIGH);
     tone(Buzzer, 333,100);
     // ========== Attendre le relacher du B.P. central. ==========
     while (analogRead(BP_central) < 512); // Attendre le relâché.
     digitalWrite(BLEUE, LOW); digitalWrite(VERTE, LOW); digitalWrite(ROUGE, LOW);
     // ===========================================================
     BP_central_a_traiter = true;} }

void TRAITE_LA_ROTATION_DU_CODEUR_INCREMENTAL() {
  if (Rotation) {if (Sens_positif) Compteur ++; else Compteur --;}
  Rotation = false; INT_active = true;}
  // Ci-dessus : Armer un nouveau comptage et Prise en compte du traitement.


